import { BaseSDK } from './sdk';
import { TaskTracker, TaskResponse } from '@jolibox/types';

export class TaskTrackerSDK extends BaseSDK implements TaskTracker {
  async onLevelFinished(levelId: string | number, result: boolean, duration: number): Promise<TaskResponse> {
    const errMsg = this.canIUseIfThrow('TaskTrackerSDK.onLevelFinished');
    if (errMsg) {
      return Promise.resolve(errMsg);
    }
    return await this.commands.executeCommand('TaskTrackerSDK.levelFinished', levelId, { result, duration });
  }
  async onTaskFinished(taskId: string | number, duration: number): Promise<TaskResponse> {
    const errMsg = this.canIUseIfThrow('TaskTrackerSDK.onTaskFinished');
    if (errMsg) {
      return errMsg;
    }
    return await this.commands.executeCommand('TaskTrackerSDK.taskFinished', taskId, { duration });
  }
  async onTaskEvent(
    taskId: string | number,
    params: {
      tools?: {
        id: string | number;
        name: string;
        count?: number;
        description?: string;
        price?: {
          amount: number;
          unit: string;
        };
      }[];
      awards?: {
        id: string | number;
        name: string;
      }[];
    }
  ): Promise<TaskResponse> {
    const errMsg = this.canIUseIfThrow('TaskTrackerSDK.onTaskEvent');
    if (errMsg) {
      return errMsg;
    }
    return await this.commands.executeCommand('TaskTrackerSDK.taskEvent', taskId, params);
  }
  async onLevelUpgrade(levelId: string | number, name: string): Promise<TaskResponse> {
    const errMsg = this.canIUseIfThrow('TaskTrackerSDK.onLevelUpgrade');
    if (errMsg) {
      return errMsg;
    }
    return await this.commands.executeCommand('TaskTrackerSDK.levelUpgrade', levelId, name);
  }
  async onHistoryUserLevel(level: number): Promise<TaskResponse> {
    const errMsg = this.canIUseIfThrow('TaskTrackerSDK.onHistoryUserLevel');
    if (errMsg) {
      return errMsg;
    }
    return await this.commands.executeCommand('TaskTrackerSDK.historyUserLevel', level);
  }
  async onHistoryUserScore(score: number): Promise<TaskResponse> {
    const errMsg = this.canIUseIfThrow('TaskTrackerSDK.onHistoryUserScore');
    if (errMsg) {
      return errMsg;
    }
    return await this.commands.executeCommand('TaskTrackerSDK.historyUserScore', score);
  }
}
